// $Id: TCPPacket.java,v 1.10 2001/06/05 07:02:23 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.net;

import net.sourceforge.jpcap.util.AnsiEscapeSequences;


/**
 * A TCP packet.
 * <p>
 * Extends an IP packet, adding a TCP header and TCP data payload.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.10 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/06/05 07:02:23 $
 */
public class TCPPacket extends IPPacket 
{
  /**
   * Create a new TCP packet.
   */
  public TCPPacket(int lLen, byte [] bytes,
                   String macSourceAddress, String macDestinationAddress, 
                   String sourceAddress, int sourcePort,
                   String destinationAddress, int destinationPort,
                   int ipProtocol) {
    super(lLen, bytes, 
          macSourceAddress, macDestinationAddress, 
          sourceAddress, destinationAddress,
          IPProtocols.TCP);

    int offset = lLen + IPEncoding.HEADER_LEN;
    int headerLen = TCPEncoding.HEADER_LEN;
    this.header = PacketEncoding.extractHeader(offset, headerLen, bytes);
    this.data = PacketEncoding.extractData(offset, headerLen, bytes);
    this.sourcePort = sourcePort;
    this.destinationPort = destinationPort;
  }

  public int getSourcePort() {
    return sourcePort;
  }

  public int getDestinationPort() {
    return destinationPort;
  }

  /**
   * Fetch the tcp header, excluding tcp data payload.
   */
  public byte [] getHeader() {
    return header;
  }

  /**
   * Fetch data portion of the tcp header.
   */
  public byte [] getData() {
    return data;
  }

  /**
   * Convert this TCP packet to a readable string.
   */
  public String toString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append('[');
    buffer.append(AnsiEscapeSequences.YELLOW);
    buffer.append("TCPPacket");
    buffer.append(AnsiEscapeSequences.RESET);
    buffer.append(": ");
    buffer.append(sourceAddress);
    buffer.append('.');
    buffer.append(IPPort.getName(sourcePort));
    buffer.append(" -> ");
    buffer.append(destinationAddress);
    buffer.append('.');
    buffer.append(IPPort.getName(destinationPort));
    buffer.append(" l=" + header.length + "," + data.length);
    buffer.append(']');

    return buffer.toString();
  }


  // todo: specify all of the tcp header fields here. many are missing..

  int sourcePort;

  int destinationPort;

  /**
   * TCP header.
   */
  byte [] header;

  /**
   * TCP data.
   */
  byte [] data;

  private String _rcsid = 
    "$Id: TCPPacket.java,v 1.10 2001/06/05 07:02:23 pcharles Exp $";
}
